#!/usr/bin/python3
# -*- coding: utf-8 -*-

##imports
#stdlib
import json
import os
import collections
import shutil
import getpass
#self
from lib.tools import get_soup
from lib.text import *
#other
from bs4 import BeautifulSoup


##class
class DB:
    """
    Behaves like a meta-dict : getitem function will work the same,
    but the DB has another attributes and methods to build lists as
    query answers.
    """

    ##init and overcharge
    def __init__(self, path_to_db = '/var/lib/git-pm/database.json', flush=False):
        self.path = path_to_db
        if not os.path.isdir(os.path.dirname(self.path)) and not flush:
            os.mkdir(os.path.dirname(self.path))
            os.chmod(os.path.dirname(self.path) , 0o775)
        if not os.path.isfile(self.path): #creates a db if there is none
            self.db = {"althonos/git-pm": {
                              "abs_name": "althonos.git-pm",
                              "author":"althonos",
                              "install_type": "setup.py",
                              "installed": "",
                              "installed_files": [],
                              "latest": "",
                              "name":"git-pm",
                              "state":"rolling",
                              "url":"http://github.com/althonos/git-pm"
                                }
                      }
            if not flush:
                self.dump()
                os.chmod(self.path , 0o666)
        else:
            d = json.decoder.JSONDecoder()
            with open(self.path, 'r') as db_file:
                self.db = d.decode(db_file.read()) #decodes the db


    def __contains__(self,token): #checks for an occurence of token in subdicts values
        for repo in self.db.keys():
            if token == repo: return True
            for var in self.db[repo].keys():
                if token == self.db[repo][var]: return True
        return False


    ## utilities
    def dump(self):
        if getpass.getuser() == 'root':
            with open(self.path, 'w') as db_file:
                e = json.encoder.JSONEncoder(sort_keys=True, indent=3)
                db_file.write(e.encode(self.db))

    def freeze(self, package):
        package = self.getRepo(package)
        self.db[package]['state'] = 'frozen'

    def roll(self, package):
        package = self.getRepo(package)
        self.db[package]['state'] = 'rolling'

    def update(self, package):
        print('', end='')
        package_soup = get_soup(self.db[package]['url'])
        commit = package_soup.find('a', {'class':'commit-tease-sha'}).get_text().split()[0]
        if self.db[package]['latest'] != commit:
            self.db[package]['latest'] = commit
            return False
        return True

    def register(self, wizard, options):
        package_dict = {"author" : wizard.author,
                        "name" : wizard.name,
                        "abs_name" : wizard.absolute_name,
                        "installed": wizard.latest_commit,
                        "latest": wizard.latest_commit,
                        "state": options['setup']['defaultstate'],
                        "url": wizard.url,
                        "install_type": wizard.install_type,
                        "installed_files": wizard.installed_files}
        self.db[wizard.git_name] = package_dict

    def deregister(self, package):
        del(self.db[package])

    def flush(self):
        for package in self.getAllRepos():
            if self.getRepo(package) != 'althonos/git-pm':
                del(self.db[package])
        self.dump()

    ##attr acces
    def getReposByAuthor(self, author):
        return [repo for repo in self.db.keys() if self.db[repo]['author'] == author]

    def getAllRepos(self, options=None):
        return [repo for repo in self.db.keys()]

    def getReposByState(self, state):
        return [repo for repo in self.db.keys() if self.db[repo]['state'] == state]

    def getRepoByUrl(self, url):
        for repo in self.db.keys():
            if self.db[repo]['url'] == url:
                return repo

    def getRepo(self, name):
        for repo in self.db.keys():
            if self.db[repo]['name'] == name:
                return repo
            if repo == name:
                return repo
            if self.db[repo]['url'] == name:
                return repo

    def getReposUpToDate(self, uptodate):
        if uptodate:
            return [repo for repo in self.db.keys() if self.db[repo]['installed'] == self.db[repo]['latest']]
        else:
            return [repo for repo in self.db.keys() if self.db[repo]['installed'] != self.db[repo]['latest']]


    def getRepoDetails(self, repo):
        #must be in format auth/name
        return self.db[repo]

    def getAllAuthors(self):
        return [self.db[repo]['author'] for repo in self.db.keys()]

    def getAllNames(self):
        return [self.db[repo]['name'] for repo in self.db.keys()]

    #filter
    def getFilteredRepos(self, options):

        package_list_by_state = []
        package_list_by_author = []
        package_list_by_release = []

        if options['packages'] is not None:
            package_list = options['packages']
        else:
            package_list = self.getAllRepos()

        if options['author'] is not None:
            for author in options['author']:
                if author not in self.getAllAuthors():
                    options['parser'][options['cmd']].error('no packages of author {} in database'.format(author))
                package_list_by_author += self.getReposByAuthor(author)
        else:
             package_list_by_author = package_list

        if options['rolling'] is not None:
            package_list_by_state += self.getReposByState('rolling')
        elif options['frozen'] is not None:
            package_list_by_state += self.getReposByState('frozen')
        else:
            package_list_by_state = package_list

        if options['uptodate'] is not None:
            package_list_by_release += self.getReposUpToDate(options['uptodate'])
        else:
            package_list_by_release = package_list

        return [x for x in package_list if (x in package_list_by_author) and (x in package_list_by_release) and (x in package_list_by_state)]
